/********************************************************************/
/* BGIVIEW: displays viewport output on a BGI screen.               */
/********************************************************************/

#include <bgiview.h>
#include <graphics.h>

/**************************** CBgiView:: ****************************/

/**************************** constructor ***************************/

CBgiView::CBgiView()
{
	inited = FALSE;
	bottom = 0;
}

/**************************** destructor ****************************/

CBgiView::~CBgiView()
{
	ExitBGI();
}

/****************************** methods *****************************/

/********************************************************************/
/* Size: returns viewport size.                                     */
/********************************************************************/

FP2D CBgiView::Size()
{
	InitBGI();
	return FP2D(getmaxx(), getmaxy());
}

/********************************************************************/
/* Line: draws a line on the BGI viewport.                          */
/********************************************************************/

void CBgiView::Line(const FP2D &P, const FP2D &Q)
{
	POINT iP = (POINT)P, iQ = (POINT)Q;

	InitBGI();
	line(iP.x, bottom - iP.y, iQ.x, bottom - iQ.y);
}

/********************************************************************/
/* InitBGI: initializes BGI (when inited is FALSE).                 */
/********************************************************************/

void CBgiView::InitBGI()
{
	if(!inited)
	{
		char far *env = envstr("BGI");
		int card = DETECT, mode;

		initgraph(&card, &mode, env ? env : "");
		inited = TRUE;
		bottom = getmaxy();
	}
}

/********************************************************************/
/* ExitBGI: terminates BGI session (when inited is TRUE).           */
/********************************************************************/

void CBgiView::ExitBGI()
{
	if(inited)
	{
		closegraph();
		inited = FALSE;
	}
}
