/********************************************************************/
/* DXFIN: DXF file input.                                           */
/********************************************************************/

#ifndef __DXFIN_INCLUDED__
#define __DXFIN_INCLUDED__

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <trc.h>

enum { RT_STR, RT_INT, RT_FLT, RT_LNG };

enum
{
	DXF_EOF   = -1,
	DXF_ERROR = -100,
	DXF_FILE,   /* file failed to open */
	DXF_FORMAT, /* invalid DXF file format */
	DXF_GROUP,  /* group code expected */
	DXF_INT,    /* integer value expected */
	DXF_FLT,    /* floating point value expected */
	DXF_LINE,   /* line too long */
	DXF_EXTRA,  /* extra characters on line */
};

struct TDxfLabel
{
	long line;
	long addr;
	BOOL used;
};

class CDxfin
{
public:
	enum { MAX_STRING = 256 };

	FILE  *fp;     /* NULL = file is not opened */
	BOOL   binary; /* TRUE = binary DXF */
	long   size;   /* file size */
	long   addr;   /* file pos of record beginning */
	long   line;   /* line pos of record beginning (ASCII DXF only) */
	long   lcnt;   /* current line number (ASCII DXF only) */
	BOOL   used;   /* TRUE = current record has already been used */
	char   strVal[MAX_STRING];  /* string record value */
	double fltVal;              /* real record value */
	int    intVal;              /* integer record value */
	long   lngVal;              /* long integer record value */
	int    type;   /* record type (RT_...) */
	int    group;  /* group type */

	CDxfin();
	virtual ~CDxfin();

	BOOL Open(const char *fn);
	BOOL Open(FILE *fp);
	virtual void Close();

	int  Fetch();  /* reads 1 pair, -1 -> EOF */
	int  Record(); /* reads next when used, returns group / -1 on EOF */
	void Use() { used = TRUE; } /* marks record as used */
	BOOL Seek(int group); /* seeks given group code; FALSE on EOF */
	BOOL Str(int group, const char *value); /* seeks string; marks as used */
	BOOL Eof() { return group < 0; }

	BOOL Section(); /* check group 0 code SECTION */
	BOOL EndSec();  /* check group 0 code ENDSEC */

	void GetPos(TDxfLabel &lbl); /* stores DXF pos */
	void SetPos(const TDxfLabel &lbl); /* moves DXF pos to given address */

	const char *ErrName(); /* returns error description string */
	long        ErrAddr(); /* returns error address/line */

	int         Done();    /* returns percentage of file processed */

	void PrintOn(FILE *stream); /* displays current record */

private:
	void FetchAsc();
	void FetchBin();
	BOOL InitRead();
	BOOL SkipEol(int c1);
};

#endif/*__DXFIN_INCLUDED__*/