/********************************************************************/
/* DXFREAD: high-level DXF input.                                   */
/********************************************************************/

#ifndef __DXFREAD_INCLUDED__
#define __DXFREAD_INCLUDED__

#include <dxfin.h>
#include <geometry.h>
#include <shxfont.h>

class C_BlockList
{
public:
	C_BlockList *link;
	FP2D         ins;     // insertion point
	TDxfLabel    addr;    // address in file
	char         name[1]; // name

	C_BlockList(C_BlockList *cdr, TDxfLabel addr, const char *name);
};

struct CDxfStack
{
public:
	CDxfStack   *link;
	C_BlockList *block;
	CMatrix     orig;
	int         row, row_count;
	int         col, col_count;
	double      angle;
	FP2D        ins;
	FP2D        spacing;
	FP2D        scale;
	TDxfLabel   addr;

	CDxfStack(CDxfStack *join);
	CMatrix Matrix();
};

struct CPathList
{
public:
	CPathList  *link;
	char        path[1];

	static CPathList *Make(CPathList *join, const char far *path);
};

class C_StyleList
{
public:
	C_StyleList *link;
	CShxDraw    *shx;
	char         name[1];

	C_StyleList(C_StyleList *cdr, CShxDraw *shx, const char *style_name);
	CShxDraw *Style(const char *name);
	void Delete();
};

class CDxfReader : public CDxfin
{
protected:
	C_BlockList *blocks;
	CDxfStack   *stack;
	C_StyleList *styles;
	CPathList   *paths;
	TDxfLabel    entities;
	BOOL         scanned;
	BOOL         raw_text;

	BOOL Scan();
	void EnterBlock();
	void LeaveBlock();

	void StackPush();
	void StackPop();
	void KillStack();

	C_BlockList *Block(const char *name);

	void  ExpandText();
	void  ExpandCircle();
	void  ExpandPoint();
	void  ExpandLine();
	void  ExpandPolyline();
	void  ExpandArc();

public:
	CMatrix transform;
	CTransformedView view;

	// maintenance
	CDxfReader();
	virtual ~CDxfReader();

	// output viewport selection & inquiry
	void OnView(CViewport *_view) { view.View(_view); }
	CViewport *View() { return view.View(); }

	// style routines
	void      AddPath(const char far *path);
	CShxDraw *Style(const char *name);

	// text control
	void RawText(BOOL raw_flag) { raw_text = raw_flag; }

	virtual void Close();
	virtual BOOL OnEntity();

	BOOL Expand(void (*progress)(int) = NULL);
};

#endif/*__DXFREAD_INCLUDED__*/