/********************************************************************/
/* DYNAMEM: dynamic memory manager.                                 */
/********************************************************************/

#include <dynamem.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>

void *cdecl operator new(size_t size)
{
	return CHeap::Alloc(size);
}

void *cdecl operator new(size_t size, size_t extra)
{
	return CHeap::Alloc(size + extra);
}

void *cdecl operator new(size_t size, const char *extra)
{
	return CHeap::Alloc(size + strlen(extra));
}

void  cdecl operator delete(void *ptr)
{
	CHeap::Free(ptr);
}

/****************************** CHeap:: *****************************/

/****************************** methods *****************************/

/********************************************************************/
/* Alloc: allocates memory on heap. Exits on error.                 */
/********************************************************************/

void *CHeap::Alloc(size_t size)
{
	void *ptr;

	if((ptr = malloc(size)) == NULL)
	{
		fprintf(stderr, "Fatal error: not enough memory\n");
		exit(-1);
	}

	return ptr;
}

/********************************************************************/
/* Free: frees previously allocated memory.                         */
/********************************************************************/

void CHeap::Free(void *p)
{
	if(p)
		free(p);
}
