/********************************************************************/
/* ENTITY: DXF entity processors.                                   */
/********************************************************************/

#include <entity.h>
#include <stdio.h>


/****************************** macros ******************************/

#define ENTRY(cl, name) \
	CEntity *cl::New() { return new cl; }; \
	ALIAS(cl, name)

#define ALIAS(cl, name) static CEntityList __##name##_stub__(#name, cl::New);

/********************* private class CEntityList ********************/

class CEntityList
{
public:
	const char  *name;
	CEntity     *(*New)();
	CEntityList *link;

	static CEntityList *entityList;

	CEntityList(const char *name, CEntity *(*New)());

	static CEntity *Make(const char *name);
};

/*************************** CEntityList:: **************************/

/**************************** static data ***************************/

CEntityList *CEntityList::entityList = NULL;

/**************************** constructor ***************************/

CEntityList::CEntityList(const char *_name, CEntity *(*_new)())
{
	name       = _name;
	New        = _new;
	link       = entityList;
	entityList = this;
}

/****************************** methods *****************************/

/********************************************************************/
/* Make: creates entity according to name.                          */
/********************************************************************/

CEntity *CEntityList::Make(const char *name)
{
	for(CEntityList *list = entityList; list; list = list -> link)
		if(!strcmp(list -> name, name))
			return list -> New();
	return NULL;
}

/***************************** CEntity:: ****************************/

/**************************** destructor ****************************/

CEntity::~CEntity() {}

/****************************** methods *****************************/

/********************************************************************/
/* Get: reads entity from DXF stream. Returns 0 when unknown.       */
/********************************************************************/

CEntity *CEntity::Get(CDxfin &stream)
{
	if(stream.Seek(0) && !stream.EndSec())
	{
		CEntity *ent;

		if((ent = CEntityList::Make(stream.strVal)) != NULL)
		{
			stream.Use();
			ent -> Read(stream);
			return ent;
		}
	}
	return NULL;
}

/**************************** CEntLine:: ****************************/

ENTRY(CEntLine, LINE)
ALIAS(CEntLine, 3DLINE)

/****************************** methods *****************************/

/********************************************************************/
/* Read: reads the entity from a DXF stream.                        */
/********************************************************************/

void CEntLine::Read(CDxfin &stream)
{
	stream.Use();
}

/********************************************************************/
/* DrawOn: draws entity on given output device.                     */
/********************************************************************/

void CEntLine::DrawOn(CViewport &view)
{
	view.MoveTo(FP2D::O);
}

/********************************************************************/
/* PrintOn: prints entity information on given stream.              */
/********************************************************************/

void CEntLine::PrintOn(FILE *stream)
{
	fprintf(stream, "LINE ");
	line.PrintOn(stream);
	fputc('\n', stream);
}

/********************************************************************/
/* Transform: transforms the entity through a matrix.               */
/********************************************************************/

void CEntLine::Transform(CMatrix &matrix)
{
	line = line * matrix;
}
