/********************************************************************/
/* FP: real coordinate 2D & 3D vector.                              */
/********************************************************************/

#ifndef __FP_H
#define __FP_H

#include <trc.h>
#include <float.h>

extern const double NIL;

class FP2D
{
public:
	double x, y;

	FP2D()                     { x = y = 0; }
	FP2D(const FP2D &p)        { x = p.x; y = p.y; }
	FP2D(double _x, double _y) { x = _x; y = _y; }
	FP2D(POINT p)              { x = p.x; y = p.y; }
	FP2D(LONG p)               { x = (int)LOWORD(p); y = (int)HIWORD(p); }

	operator POINT () const;
	operator LONG  () const;

	FP2D  operator + (const FP2D &) const;
	FP2D  operator - (const FP2D &) const;
	FP2D  operator * (double) const;
	FP2D  operator / (double) const;

	friend FP2D operator * (double x, const FP2D &p) { return p * x; }

	FP2D &operator += (const FP2D &);
	FP2D &operator -= (const FP2D &);
	FP2D &operator *= (double);
	FP2D &operator /= (double);

	FP2D &operator <= (const FP2D &);
	FP2D &operator >= (const FP2D &);

	double operator * (const FP2D &) const; /* scalar product */
	double operator % (const FP2D &) const; /* determinant    */
	double operator | (const FP2D &) const; /* distance       */
	double operator ^ (const FP2D &) const; /* projection     */

	int operator == (const FP2D &pt) const;
	int operator != (const FP2D &pt) const { return !(*this == pt); }

	FP2D scale  (const FP2D &centre, double to)    const;
	FP2D rotate (const FP2D &centre, double angle) const;
	FP2D left   () const { return FP2D(-y, x); }
	FP2D right  () const { return FP2D(y, -x); }
	FP2D unit   () const;
	FP2D middle (const FP2D &B) const;

	double bearing () const;
	double squared () const;
	double length ()  const;

	BOOL   IsNull() const;

	static FP2D Polar(double r, double a);

	static FP2D O;
	static FP2D NIL;
};

class FP3D
{
public:
	double x, y, z;

	FP3D()                                { x = y = z = 0; }
	FP3D(const FP3D &p)                   { x = p.x; y = p.y; z = p.z; }
	FP3D(double _x, double _y, double _z) { x = _x; y = _y; z = _z; }

	FP3D  operator + (const FP3D &) const;
	FP3D  operator - (const FP3D &) const;
	FP3D  operator % (const FP3D &) const;
	FP3D  operator * (double) const;
	FP3D  operator / (double) const;

	friend FP3D operator * (double x, const FP3D &p) { return p * x; }

	FP3D &operator += (const FP3D &);
	FP3D &operator -= (const FP3D &);
	FP3D &operator *= (double);
	FP3D &operator /= (double);

	FP3D &operator <= (const FP3D &);
	FP3D &operator >= (const FP3D &);

	double operator * (const FP3D &) const; /* scalar product */
	double operator | (const FP3D &) const; /* distance       */
	double operator ^ (const FP3D &) const; /* projection     */

	int operator == (const FP3D &pt) const;
	int operator != (const FP3D &pt) const { return !(*this == pt); }

	double mix ( const FP3D &p, const FP3D &q) const;
	/* mixed product = determinant */

	FP3D scale  (const FP3D &centre, double to) const;
	FP3D unit   () const;
	FP3D middle (const FP3D &B) const;

	double squared () const;
	double length () const;

	BOOL   IsNull() const;

	static FP3D O;
	static FP3D NIL;
};

#endif/*__FP_H*/