/********************************************************************/
/* HIFIN: a single buffered input file.                             */
/********************************************************************/

#ifndef __HIFIN_H
#define __HIFIN_H

#define INP_BUF 10000

extern int pascal fip, pascal fieof;
extern char pascal fin[80];
extern long pascal fisize, pascal fitell;
extern unsigned char pascal fibuf[INP_BUF], pascal *fiptr, pascal *fiend;

#ifdef __cplusplus
#define proc(type) extern "C" type pascal
#else
#define proc(type) type pascal
#endif
proc(void) i_close(void);
/* Closes the file; when it is not opened, does nothing but must behave  */
/* correctly (multiple closes and closes of unopened files are allowed). */
proc(int) i_open(const void far *fn);
/* Opens a file for read. Fills out fisize with its length. Returns 0=OK, */
/* nonzero on error. */
proc(void) i_join(const void far *name, int fp);
/* Allows buffered access to an already opened file. <fp> must be a valid */
/* file handle. name is used only to copy to <fin>. File length is checked */
/* and stored into <fisize>. */
proc(int) i_fill(void);
#define i_getc() (fiptr == fiend ? i_fill() : (int)*fiptr++)
/* Reads one character from the stream. The stream must be opened */
/* (otherwise the results are undefined). On EOF, -1 is returned. */
#define i_ungetc() (fieof ? (void)0 : (void)fiptr--)
/* Un-reads one character from input stream. Does nothing on EOF. */
proc(int) i_getw(void);
/* Reads a word (low-order byte first) from the file. When 2nd byte or */
/* both bytes are past EOF, result is undefined. */
proc(long) i_getl(void);
/* Reads a long word (lowest byte first) from the stream. When a part of */
/* the number is past EOF, the result is undefined. */
proc(int) i_getmw(void);
/* Reads a Motorola word (high-order byte first) from the file. When 2nd
byte or both bytes are past EOF, result is undefined. */
proc(long) i_getml(void);
/* Reads a Motorola long word (highest byte first) from the stream. When
a part of the number is past EOF, the result is undefined. */
proc(int) i_read(void far *buf, int size);
/* Reads <size> bytes into buffer <buf> and returns number of bytes */
/* actually read (0 on EOF). */
proc(int) i_aread(void far *buf, int size);
/* Reads <size> bytes into buffer <buf>. Returns 0=OK - all has been read, */
/* nonzero on EOF. */
proc(long) i_lread(void far *buf, long size);
/* Reads <size> byets into buffer <buf>. */
proc(int) i_alread(void far *buf, long size);
/* Reads <size> bytes into buffer <buf>. Returns 0=OK - all has been read, */
/* nonzero on EOF. */
proc(long) i_tell(void);
/* Returns current position of file pointer. */
proc(void) i_seek(long to);
/* Moves the file pointer to an absolute location relative to the beginning */
/* of the file. */
proc(void) i_rseek(int shift);
/* Moves the file pointer by <shift> bytes (positive = forward). */
proc(void) i_rewind(void);
/* Moves the file pointer to the beginning of file. */
proc(int) i_lopen(const void far *fn);
/* Opens the file for line-oriented access. <filx> is initialized to 0. */
extern long pascal filx; /* current line number */
proc(int) i_line(void);
/* Reads one line from the input stream into internal static buffer <fil>. */
/* Returns -1 = EOF, 0 = OK, +1 = line too long (maximum 1000 characters). */
#define MAX_INPUT_LINE 1000
extern unsigned char pascal fil[MAX_INPUT_LINE];
extern int pascal filsz; /* strlen of currently read input line */
#endif/*__HIFIN_H*/
/********************************************************************/
/* STRINGS: string manipulation functions.                          */
/********************************************************************/

#ifndef __STRINGS_H
#define __STRINGS_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NULL
#define NULL 0
#endif

char *pascal strcpy(void *to, const void *from);
char far *pascal _fstrcpy(void far *to, const void far *from);
/* copies <from> to <to>, returns <to>. */

char *pascal stpcpy(void *to, const void *from);
char far *pascal _fstpcpy(void far *to, const void far *from);
/* copies <from> to <to>, returns <to> + strlen(from). */
char *pascal strcat(void *to, const void *from);
char far *pascal _fstrcat(void far *to, const void far *from);
/* appends <from> to <to>, returns <to>. */
char *pascal chrcat(void *to, char x);
char far *pascal _fchrcat(void far *to, char x);
/* appends character <x> to <to>, returns <to>. */
char *pascal strwhite(const void *s);
char far *pascal _fstrwhite(const void far *s);
/* skips all blank characters in string s. Returns pointer to 1st nonblank */
/* character or to terminating 0. */
char pascal last(const void *s);
char pascal _flast(const void far *s);
/* returns last character in string or 0 when it is empty. */
char pascal cut(void *s);
char pascal _fcut(void far *s);
/* returns last character in string and rewrites it with zero (decrements */
/* string length). */
unsigned pascal strlen(const void *s);
unsigned pascal _fstrlen(const void far *s);
/* Returns length of string <s>. */
char *pascal strend(const void *s);
char far *pascal _fstrend(const void far *s);
/* Returns s + strlen(s), the pointer to terminating 0 */
char *pascal strechr(const void *s, char c);
char far *pascal _fstrechr(const void far *s, char c);
/* Returns position in <s> where <c> occures for the 1st time or pointer */
/* to terminating 0 when <c> is not present in <s>. */
int pascal strcmp(const void *s1, const void *s2);
int pascal _fstrcmp(const void far *s1, const void far *s2);
/* returns -1 .. s1 < s2, 0 .. s1 == s2, +1 .. s1 > s2. Comparation is */
/* unsigned. */
char *pascal substrcmp(const void *s1, const void *s2);
char far *pascal _fsubstrcmp(const void far *s1, const void far *s2);
/* if the beginning of string <s1> is equal to <s2>, points to s1 */
/* + strlen(s2). If s2 is not present on the beginning of s1, returns NULL. */
void pascal strlwr(void *s);
void pascal _fstrlwr(void far *s);
/* Changes characters 'A' - 'Z' to 'a'-'z' within s. */
void pascal memcpy(void *to, const void *from, unsigned size);
void pascal _fmemcpy(void far *to, const void far *from, unsigned size);
/* copies <size> bytes from <from> to <to>. Blocks may not overlap. Segment */
/* wraparound is not handled. */
void pascal memmove(void *to, const void *from, unsigned size);
void pascal _fmemmove(void far *to, const void far *from, unsigned size);
/* copies <size> bytes from <from> to <to>. Blocks may overlap, but the */
/* result of moving overlapping blocks may be incorrect when <to> and <from> */
/* are in different segments (i.e. only offsets are compared to determine */
/* the direction). */
void pascal memset(void *to, char value, int num);
void pascal _fmemset(void far *to, char value, int num);
/* fills <num> bytes at address <to> with <value>. */
int pascal memcmp(const void *x, const void *y, unsigned len);
int pascal _fmemcmp(const void far *x, const void far *y, unsigned len);
/* compares <len> bytes of <x> and <y>; returns -1 .. x < y, 0 .. x == y, */
/* 1 .. x > y. Comparison is unsigned. */
unsigned pascal strnum(const void *s, char c);
unsigned pascal _fstrnum(const void far *s, char c);
/* Returns offset of 1st occurence of <c> within <s> or strlen(s) when */
/* s does not contain any <c>. */
char *pascal strmcpy(void *to, const void *from, int space);
char far *pascal _fstrmcpy(void far *to, const void far *from, int space);
/* copies <from> to <to>, but space-1 characters at most; together with */
/* the terminating zero the <to> string will occupy <space> bytes at most. */
void pascal strmcat(void *to, const void *from, int space);
void pascal _fstrmcat(void far *to, const void far *from, int space);
/* appends <from> to <to>; together with the terminating zero the <to> string */
/* will occupy <space> bytes at most. */
char *pascal strlist(const void *s, int n);
char far *pascal _fstrlist(const void far *s, int n);
/* finds nth item in a series of strings separated by 0's. */
void pascal memswap(void *a, void *b, unsigned nrof);
void pascal _fmemswap(void far *a, void far *b, unsigned nrof);
/* swaps <nrof> bytes at adresses <a> and <b>. Blocks may not overlap. */
/* Segment wraparound is not handled. */
void *pascal memchr(const void *s, char c, unsigned len);
void far *pascal _fmemchr(const void far *s, char c, unsigned len);
/* Finds 1st occurence of <c> in the 1st <len> bytes of <s>. Returns NULL */
/* when <c> was not found. */
int pascal memupd(void *to, const void *from, unsigned size);
int pascal _fmemupd(void far *to, const void far *from, unsigned size);
/* copies <size> bytes from <from> to <to>. Returns 0 when both arrays */
/* had same values prior to copying (when copying was unnecessary). */
void pascal farmove(void far *to, const void far *from, long size);
/* copies <size> bytes from <from> to <to>. Blocks may overlap. This is */
/* not Windows compatible. */
void pascal wfarmove(void far *to, const void far *from, long len);
/* copies <size> bytes from <from> to <to>. Blocks may overlap. This */
/* function is Windows compatible (but the result may not be correct, if */
/* two different selectors were used to point to the same physical memory */
/* area. */
void pascal wfarcpy(void far *to, const void far *from, long len);
/* copies <size> bytes from <from> to <to>. Blocks may NOT overlap. This */
/* function is Windows compatible. */

void pascal wfarset(void far *dest, char value, long len);
/* sets <size> bytes in <dest> to <value>. This is Windows compatible. */
#ifdef __cplusplus
}
#endif

#endif
/********************************************************************/
/* CRT: text-mode screen output routines.                           */
/********************************************************************/

#ifndef __CRT_H
#define __CRT_H

#ifdef __cplusplus
extern "C" {
#endif

struct display_info
{
	int xsize, ysize;       /* screen size */
	int wx, wy, ww, wh;     /* current window */
	unsigned char far *ptr; /* current address */
	unsigned nxt;           /* offset of next line */
	char mode, attrib;      /* current text mode and attribute */
};

extern struct display_info pascal crt; /* current output parameters */

void pascal crt_init(void); /* sets up crt output */
void pascal crt_exit(void); /* should be called upon exit */
void pascal gotoxy(int x, int y);
/* set cursor pos; [0,0] = topleft corner of window ([wx,wy]) */
void pascal setxy(int x, int y);
/* set absolute cursor pos; [0,0] = topleft corner of screen */
void pascal putch(char c);
/* writes a character and moves cursor; no line wrap, no scroll! Uses */
/* current attribute (crt.attrib). */
unsigned pascal getch(void);
/* reads a character & attribute from the cursor position. */
void pascal pattr(void);
/* changes attribute at cursor position to crt.attrib. Moves cursor. */
void pascal putwc(unsigned wc);
/* Writes character & attribute to cursor position. Moves cursor. */
void pascal cmove(void);
/* Just moves the text cursor 1 character to the right. No line wrapping, */
/* no scrolling! */
void pascal peoln(void);
/* Moves cursor 1 line down. X position is set to that of last gotoxy. */
void pascal cputn(char c, int num);
/* Writes num times character c to the current cursor position. Does not */
/* move cursor. No line wrapping, no scroll! */
enum cursor_type { _NOCURSOR, _SOLIDCURSOR, _NORMALCURSOR };
void pascal cursor(char c_type);
/* Sets cursor type (none / block / underscore). */
void pascal cputs(const char far *s);
/* outputs a string to the cursor position. Updates cursor position. No line */
/* wrapping, no scroll. Uses current attribute. */
void pascal gettext(int x, int y, int w, int h, void far *to);
/* Reads block of screen into memory. Block size is 2 * w * h. x and y are */
/* absolute ([0,0] is the topleft corner of the screen). */
/* No out-of-screen checks are performed. */
void pascal puttext(int x, int y, int w, int h, const void far *src);
/* Writes a memory block to screen. Block size is 2 * w * h. x and y are */
/* absolute ([0,0] is the topleft corner of the screen). */
/* No out-of-screen checks are performed. */
void pascal settext(int x, int y, int w, int h, char c);
/* Fills a screen area with given character. Current attribute is used. X */
/* and Y are absolute ([0,0] is the topleft corner of the screen). */
/* No out-of-screen checks are performed. */
void pascal movetext(int x, int y, int w, int h, int x1, int y1);
/* moves a screen window to another screen position (useful for scrolling). */
/* Source and destination area may overlap and may be the same. */
/* No out-of-screen checks are performed. */
void pascal clrscr(void);
/* Sets current attribute to 7 (white on black), clears screen and cursor. */
int pascal key0(void);
extern int pascal kbd;
/* If there is a key in the keyboard buffer, returns its ASCII code; if it */
/* has no ASCII code, returns 256 + its scan code (e.g. 0x13B is F1). If */
/* no key is pressed, returns 0. Before return it stores its return value */
/* into the global variable kbd. */
int pascal key(void);
/* Waits until a key is pressed, returns its ASCII code / scan code + 256 */
/* (see key0) and stores the code into global variable kbd. */
int pascal shifts(void);
/* Returns shift state bits. */
enum
{
	VKM_RSHIFT = 1, VKM_LSHIFT = 2, VKM_CTRL = 4, VKM_ALT = 8,
	VKM_SCROLL = 16, VKM_NUM = 32, VKM_CAPS = 64, VKM_INS = 128
};
void pascal putchr(char c);
/* Writes character c to the standard output. If c is LF (\n), */
/* CRLF is sent. */
void pascal putstr(const char far *s);
/* Writes a string to standard output. LF's are converted into CRLF. */
void pascal beep(void);
/* Calls crt BIOS to generate a beep. */
#ifdef __cplusplus
}
#endif

#endif/*__CRT_H*/
/********************************************************************/
/* FORMAT: additional text formatting routines.                     */
/********************************************************************/

#ifndef __FORMAT_H
#define __FORMAT_H

#ifdef __cplusplus
extern "C" {
#endif

int cdecl vsprintf(char *buf, const char *fmt, void *arg_list);
int cdecl sprintf(char *buf, const char *fmt, ...);
#define __FORMAT_SIZE 1000
extern char pascal __FORMAT_BUF[__FORMAT_SIZE];

char *cdecl format(const void *fmt, ...);
/* Formats given string into a local buffer. */

char *pascal vformat(const void *fmt, const void *args);
void cdecl putf(const void *fmt, ...);
#ifdef __cplusplus
}
#endif

#endif/*__FORMAT_H*/
/********************************************************************/
/* FPROC: disk-oriented functions.                                  */
/********************************************************************/

#ifndef __FPROC_H
#define __FPROC_H

#ifdef __cplusplus
extern "C" {
#endif

char *pascal epos(void *to, const void *from);
char far *pascal _fepos(void far *to, const void far *from);
/* copies filename <from> to <to>, converts to uppercase and returns pointer */
/* to <to> where file extension begins (pointer to '.' or to terminating 0 */
/* when there is no extension. */
char *pascal npos(const void *path);
char far *pascal _fnpos(const void far *path);
/* returns the pointer to the 1st character of name within filename <path>. */
char *pascal fappend(void *to, const void *from, const void *ext);
char far *pascal _ffappend(void far *to, const void far *from, const void far *ext);
/* copies filename <from> to <to>; if <from> has no extension, <ext> is */
/* appended. Returns <to>. */
char *pascal fforce(void *to, const void *from, const void *ext);
char far *pascal _ffforce(void far *to, const void far *from, const void far *ext);
/* Copies <from> to <to> and changes its extension to <ext>. Returns <to>. */
int pascal _Fcreat(const void far *name);
/* Creates file <name> with attributes 0. Returns handle or -1 on error. */
int pascal _Fopen(const void far *name, int mode);
/* Opens file <name> in mode <mode>. Returns handle or -1 on error. */
int pascal _Fclose(int handle);
/* closes file <handle>. */
int pascal _Fwrite(int handle, const void far *from, unsigned len);
/* writes <len> bytes from address <from> to file <handle>. Returns number */
/* of bytes actually written or -1 on error. Segment wraparound is not */
/* handled. */
int pascal _Fawrite(int handle, const void far *from, unsigned len);
/* Writes <len> bytes from address <from> to file <handle>. Returns 0 = OK, */
/* nonzero - error or not enough disk space. */
int pascal _Foutstr(int handle, const void far *from);
/* Writes a string to the file <handle>. Returns 0 = OK, nonzero = error. */
int pascal _Foutbyte(int handle, char x);
/* Writes a byte to file <handle>. Returns 0 = OK, nonzero = error. */
int pascal _Foutword(int handle, int x);
/* Writes a word (low byte 1st) to file <handle>. Returns 0 = OK, */
/* nonzero = error. */
int pascal _Fread(int handle, void far *to, unsigned len);
/* Reads <len> bytes from file <handle> to address <to>. Segment wraparound */
/* is not handled. Returns number of bytes actually read or -1 on error. */
int pascal _Faread(int handle, void far *to, unsigned len);
/* Reads <len> bytes from file <handle> to address <to>. Returns 0 = OK, */
/* nonzero on error or end of file. */
long pascal _Fwseek(int handle, long pos, char whence);
/* Sets new file position. whence = 0 .. from beginning, 1 .. from current */
/* position, 2 .. from end of file. Returns new file position or -1 on error. */
long pascal _Fseek(int handle, long pos);
/* Sets new absolute file position. Returns file position after seek or -1 */
/* on error. */
int pascal _Faseek(int handle, long pos);
/* Sets new absolute file position; returns 0 = OK, nonzero on error or */
/* mismatch of requested and actual file position. */
long pascal _Ftell(int handle);
/* Returns current file position or -1 on error. */
long pascal _Fsize(int handle);
/* Returns file size or -1 on error. */
int pascal _Fsfirst(const void far *path, void far *block, int attrib);
/* An equivalent of findfirst. Returns 0 = OK, -1 = no match / error. Block */
/* points to the standard ffblk structure. */
int pascal _Fsnext(void far *block);
/* An equivalent of findnext. Returns 0 = OK, -1 = no match / error. Block */
/* points to the standard ffblk structure. */
int pascal _Fdelete(const void far *fn);
/* Erases file <fn>. Returns 0 = OK, nonzero on error. */
int pascal _Fgetdir(int drive, void far *dest);
/* Reads directory of drive <drive> into <dest>. Returns 0 = OK, -1 = error. */
int pascal _Fgetdisk(void);
/* Returns current disk number (0 = A). */
void pascal _Fsetdisk(int disk);
/* Sets new active disk number. */
int pascal _Frename(const void far *what, const void far *to);
/* Renames file <what> to <to>. Returns 0 = OK, -1 = error. */
int pascal fexpand(void far *to, const void far *from);
/* Expands path <from> into complete path <to>. <to> should be 80 bytes long. */
/* Returns 0 = OK or -1 on error. */
long pascal _Flread(int handle, void far *to, long len);
/* Reads <len> bytes from file <handle> to address <to>. It is able to read */
/* more than 64K at once and is Windows protected mode compatible. Returns */
/* number of bytes actually read. */
int pascal _Falread(int handle, void far *to, long len);
/* Reads <len> bytes from file <handle> to address <to>. It is able to read */
/* more than 64K at once and is Windows protected mode compatible. Returns */
/* 0 = OK, all was read / nonzero on error or EOF. */
long pascal _Flwrite(int handle, const void far *from, long len);
/* Writes <len> bytes from address <from> to file <handle>. It is able to */
/* write more than 64K at once and is Windows protected mode compatible. */
/* Returns number of bytes actually written. */
int pascal _Falwrite(int handle, const void far *from, long len);
/* Writes <len> bytes from address <from> to file <handle>. It is able to */
/* write more than 64K at once and is Windows protected mode compatible. */
/* Returns 0 = OK, all was written / nonzero on error or disk full. */

void far * pascal malload(const char far *fn, int add0, long far *plen);
/* reads file from path <fn> into a block of memory it allocates. <add0> */
/* more bytes are allocated and zero filled above the end of the file. Actual */
/* file size is put into <plen>. This function is not Windows compatible. */
#ifdef __cplusplus
}
#endif

#endif
/********************************************************************/
/* MOUSE: mouse cursor tracking routines.                           */
/********************************************************************/

#ifndef __MOUSE_H
#define __MOUSE_H

#ifdef __cplusplus
extern "C" {
#endif

int pascal minit(int lim_x, int lim_y);
/* sets up mouse (lim_x and lim_y are maximum allowed coordinates); centre */
/* is set as initial mouse position. */
extern int pascal mousex, pascal mousey, pascal mousek, pascal mouseb;
/* [mousex,mousey] = current mouse coordinates. They are always within limits */
/* <0, lim_x>, resp. <0, lim_y>. mouseb is the current state of mouse */
/* buttons; mousek is set to state of mouse buttons only on the leading edge */
/* ie only once during the press-release cycle, then it remains zero until */
/* button is released and pressed again. */
int pascal getmouse(void);
/* Updates mouse coordinates and button state; returns the value of mousek. */
#ifdef __cplusplus
}
#endif

#endif
/********************************************************************/
/* PC: computer-specific functions.                                 */
/********************************************************************/

#ifndef __PC_H
#define __PC_H

#ifdef __cplusplus
extern "C" {
#endif

void far *pascal pnorm(const void far *s);
/* normalizes pointer s. Offset of return value will be always less than 10h. */
/* This function is not Windows compatible. */
long pascal ptol(const void far *s);
/* Converts a far pointer to linear memory address. This function is not */
/* Windows compatible. */
long pascal avail(void);
/* Returns number of free bytes in DOS memory. This function is not */
/* Windows compatible. */
void far * pascal alloc(long size);
/* Allocates <size> bytes in DOS memory. This function is not Windows */
/* compatible. Returns NULL on error. */
void pascal mfree(void far *x);
/* Frees a block previously allocated by alloc. This function is not */
/* Windows compatible. */
void pascal far hardhook(void);
/* Critical disk interrupt handler; use setvect to change standard INT 24 */
/* to this address. The INT 21 file function that have failed will return */
/* CY set and AX set to error code #15. */
#ifdef __cplusplus
}
#endif

#endif
#define OUT_BUF 10000

extern int pascal fop, pascal foerr;
extern char pascal fon[80];
extern unsigned char pascal fobuf[OUT_BUF], pascal *foptr;
extern long fosize, fotell;

#ifdef __cplusplus
#define proc(type) extern "C" type pascal
#else
#define proc(type) type pascal
#endif
proc(int) o_close(void);
proc(void) o_flush(void);

#define o_putc(x) (foptr == (&fobuf)[1] ? o_flush() : (void)0, *foptr++ = (x))
proc(int) o_create(const void far *fn);
proc(int) o_append(const void far *fn);
proc(void) o_write(const void far *buf, int size);
proc(void) o_lwrite(const void far *buf, long size);
/* Writes <size> byets from buffer <buf>. */
proc(void) o_puts(const void far *buf);
proc(void) o_line(const void far *buf);
proc(void) o_crlf(void);
proc(void) o_putw(int x);
proc(void) o_putl(long x);
proc(void) o_putmw(int x);
proc(void) o_putml(long x);
proc(void) o_putcn(char c, int size);
#define o_zero(s) o_putcn(0, (s))
proc(long) o_tell(void);
proc(long) o_size(void);
proc(void) o_seek(long to);
proc(void) o_xseek(long to, int whence);
/********************************************************************/
/* vInit: initializes graphic mode 0x13.                            */
/********************************************************************/

#ifndef __XVGA_H
#define __XVGA_H

#undef DEFAULT
#ifdef __cplusplus
extern "C" {
#define DEFAULT(x) = x
#else
#define DEFAULT(x)
#endif

typedef       void far *_VFP;
typedef const void far *CVFP;

#define __AREA__ int x, int y, int w, int h

enum { VTEXT_X = 8, VTEXT_Y = 8 };
enum { PAL_3RGB, PAL_4RGB, PAL_3BGR, PAL_4BGR, PAL_NOSHIFT };

extern signed char pascal vSize[256];

typedef struct
{
	unsigned char _seg *vseg;
	int                 offx;
	int                 offy;
	int                 clx;
	int                 cly;
	unsigned            clw;
	unsigned            clh;
	unsigned            clo;
	char far           *cursor;
	char far           *curbuf;
	int                 oldcx;
	int                 oldcy;
	void far           *polybuf;
	void far           *linebuf;
	int                 polycnt;
	int                 plx, ply, phx, phy;
	unsigned char       attr;
	unsigned char       back;
	unsigned char       fill;
}
vga_attrib;

extern vga_attrib pascal vga;

void pascal  vInit(void);
void pascal  vExit(void);

/********************************************************************/
/* vClip: sets up clipping area.                                    */
/********************************************************************/
void pascal  vClip(int left, int top, int right, int bottom);
/********************************************************************/
/* vRClip: relative clip given by width & height.                   */
/********************************************************************/
void pascal  vRClip(int left, int top, int width, int height);
/********************************************************************/
/* vFull: switches clipping and offset off.                         */
/********************************************************************/
void pascal  vFull(void);
/********************************************************************/
/* vPushClip: stores clipping+offset and switches clipping off.     */
/********************************************************************/
void pascal  vPushClip(void far *to);
/********************************************************************/
/* vPopClip: pops clipping from buffer.                             */
/********************************************************************/
void pascal  vPopClip(CVFP from);
/********************************************************************/
/* vMove: moves image into actual video memory (A000).              */
/********************************************************************/
void pascal  vMove(void);
/********************************************************************/
/* vCls: clears the whole clipping area using background colour.    */
/********************************************************************/
void pascal  vCls(void);
/********************************************************************/
/* vSync: waits until vertical retrace.                             */
/********************************************************************/
void pascal  vSync(void);
/********************************************************************/
/* vUnsync: waits until the end of the vertical retrace.            */
/********************************************************************/
void pascal  vUnsync(void);
/********************************************************************/
/* vDSync: waits until the beginning of the next retrace.           */
/********************************************************************/
void pascal  vDSync(void);
/********************************************************************/
/* vSetRGB: sets given DAC colour. Ranges are 0-255.                */
/********************************************************************/
void pascal  vSetRGB(int colour, char R, char G, char B);
/********************************************************************/
/* vGetRGB: reads DAC colour as a long word: hi-> 0,B,G,R <-lo.     */
/********************************************************************/
long pascal  vGetRGB(int colour);
/********************************************************************/
/* vGetBGR: reads DAC colour as long word with hi->0,R,G,B<-lo.     */
/********************************************************************/
long pascal  vGetBGR(int colour);
/********************************************************************/
/* vSetPalette: programs given part of palette.                     */
/********************************************************************/
void pascal vSetPalette(CVFP from, int type,
                        int first DEFAULT(0), int count DEFAULT(256));
/********************************************************************/
/* vGetPalette: reads given part of palette.                        */
/********************************************************************/
void pascal  vGetPalette(_VFP to, int type,
                             int first DEFAULT(0), int count DEFAULT(256));
/********************************************************************/
/* vPixel: displays a pixel on given screen position.               */
/********************************************************************/
void pascal  vPixel(int x, int y, char colour);
/********************************************************************/
/* vXorPixel: xors given pixel with given colour value.             */
/********************************************************************/
void pascal  vXorPixel(int x, int y, char colour);
/********************************************************************/
/* vGetPixel: returns pixel value or vga.back when out of screen.   */
/********************************************************************/
int  pascal  vGetPixel(int x, int y);
/********************************************************************/
/* vPutc: prints character at given position. Returns right side.   */
/********************************************************************/
int  pascal  vPutc(int x, int y, char chr);
/********************************************************************/
/* vPuts: prints a string on given position.                        */
/********************************************************************/
int  pascal  vPuts(int x, int y, CVFP s);
/********************************************************************/
/* vStrSize: returns pixel length of a string. No clipping here.    */
/********************************************************************/
int  pascal  vStrSize(CVFP s);
/********************************************************************/
/* vPutsBox: prints text into an area. Clears the rest of the area. */
/********************************************************************/
void pascal  vPutsBox(int x, int y, int w, int h, CVFP s, char al);
/********************************************************************/
/* vCircle: draws a circle.                                         */
/********************************************************************/
void pascal  vCircle(int x, int y, int r);
/********************************************************************/
/* vFilledCircle: fills a circle.                                   */
/********************************************************************/
void pascal  vFilledCircle(int x, int y, int r);
/********************************************************************/
/* vLine: draws a line including both endpoints.                    */
/********************************************************************/
void pascal  vLine(int x, int y, int w, int h);
/********************************************************************/
/* vHLine: horizontal line.                                         */
/********************************************************************/
void pascal  vHLine(int x, int y, int len);
/********************************************************************/
/* vVLine: vertical line.                                           */
/********************************************************************/
void pascal  vVLine(int x, int y, int len);
/********************************************************************/
/* vBox: fills a box with current fill colour.                      */
/********************************************************************/
void pascal  vBox(int x, int y, int w, int h);
/********************************************************************/
/* vXorBox: xors given area with fill colour.                       */
/********************************************************************/
void pascal  vXorBox(int x, int y, int w, int h);
/********************************************************************/
/* vWrite: copies given image to screen.                            */
/********************************************************************/
void pascal  vWrite(int x, int y, int w, int h, CVFP s, int l DEFAULT(0));
/********************************************************************/
/* vSprite: displays a sprite ignoring colour values of 0.          */
/********************************************************************/
void pascal vSprite(int x, int y, int w, int h, CVFP s, int l DEFAULT(0));
/********************************************************************/
/* vRead: copies given image from screen to a buffer.               */
/********************************************************************/
void pascal vRead(int x, int y, int w, int h, _VFP s, int l DEFAULT(0));
/********************************************************************/
/* vRect: draws a single-colour rectangle.                          */
/********************************************************************/
void pascal  vRect(int x, int y, int w, int h, int width DEFAULT(1));
/********************************************************************/
/* vShade: draws a shaded rectangle.                                */
/********************************************************************/
void pascal vShade(int x, int y, int w, int h, char c1, char c2,
                       int t DEFAULT(1));
/********************************************************************/
/* vHide: hides the cursor.                                         */
/********************************************************************/
void pascal  vHide(void);
/********************************************************************/
/* vShow: shows the cursor.                                         */
/********************************************************************/
void pascal  vShow(int x, int y);
/********************************************************************/
/* vTriangle: displays a triangle.                                  */
/********************************************************************/
void pascal  vTriangle(int x1, int y1, int x2, int y2, int x3, int y3);
/********************************************************************/
/* vInitArea: initializes fill area.                                */
/********************************************************************/
void pascal  vInitArea(void);
/********************************************************************/
/* vPolyArea: adds a closed polygon to the fill area.               */
/********************************************************************/
void pascal  vPolyArea(int n, CVFP points);
/********************************************************************/
/* vLineArea: adds a line to the fill area.                         */
/********************************************************************/
void pascal  vLineArea(int x1, int y1, int x2, int y2);
/********************************************************************/
/* vClipArea: returns 1 when area has intersection with clip.       */
/********************************************************************/
int  pascal  vClipArea(void);
/********************************************************************/
/* vFillArea: fills current area and clears it.                     */
/********************************************************************/
void pascal  vFillArea(void);
/********************************************************************/
/* vMoveClip: moves current clip from buffer to video.              */
/********************************************************************/
void pascal  vMoveClip(void);
/********************************************************************/
/* vSize: character size table.                                     */
/********************************************************************/
/********************************************************************/
/* vFont: character set.                                            */
/********************************************************************/
#ifdef __cplusplus
}
#endif

#endif/*__XVGA_H*/
/********************************************************************/
/* PRGENV: program environment functions.                           */
/********************************************************************/

#ifndef __PRGENV_H
#define __PRGENV_H

#ifdef __cplusplus
extern "C" {
#endif

char far *pascal envstr(void far *head);
void cdecl error(const char *fmt, ...);
#ifdef __cplusplus
}
#endif

#endif/*__PRGENV_H*/
/********************************************************************/
/* MACROS: some common macros.                                      */
/********************************************************************/

#ifndef __MACROS_H
#define __MACROS_H

#define PAIR(x, y) ((BYTE)(x) + 256 * (BYTE)(y))
#define TWO(s, x, y) (*(int far *)(s) == PAIR((x), (y)))
#define THREE(s, x, y, z) (TWO((s), (x), (y)) \
	&& ((BYTE far *)(s))[2] == (BYTE)(z))
#define FOUR(s, x, y, z, w) (*(long far *)(s) \
	== PAIR((x), (y)) + 0x10000L * PAIR((z), (w)))

#define POKE2(s, x, y) (*(int far *)(s) = PAIR((x), (y)))
#define POKE3(s, x, y, z) (POKE2((s), (x), (y)), ((BYTE far *)(s))[2] = (z))

#endif/*__MACROS_H*/

proc(int) update_begin(void);
proc(void) update_area(void *ptr, int bytes);
#define update(x) (update_area(&(x), sizeof((x))))
proc(int) update_end(void);
#ifndef __XASSERT__
#define __XASSERT__

#if DEBUG
#define __leftError() error(__FILE__ ", %d: "
#define __showError(e) __leftError() e, __LINE__)
#define isTrue(x) if(x) ; else __showError("false " #x)
#define CHECK(x) if(x) ; else __showError("false " #x)
#define neverComes() __showError("invalid variant")
#define NEVER() __showError("invalid variant")
#define fnReturns(x) if(__AX == x) ; else __leftError() \
	"returns %d instead of %d", __LINE__, __AX, (x))
#define fnReturnsLong(x) if((65536L * _DX | _AX) == x) ; else __leftError() \
	"returns %ld instead of %ld", __LINE__, __AX, __DX, (long)(x))
#define returnsZero() fnReturns(0)
#define returnsNonzero() if(__AX) ; else __showError("returns zero")
#define ifDebug(x) x
#else
#define isTrue(x)
#define CHECK(x)
#define neverComes()
#define NEVER()
#define fnReturns(x)
#define fnReturnsLong(x)
#define returnsZero()
#define returnsNonzero()
#define ifDebug(x)
#endif

#endif
/********************************************************************/
/* KEYS: common scan & ASCII code definitions.                      */
/********************************************************************/

#ifndef __KEYS_H
#define __KEYS_H

enum
{
   KL_M1 = -1, KL_M2 = -2, KL_M12 = -3,

   KL_BACK = 8, KL_TAB = 9, KL_RETURN = 13, KL_ESC = 27,
   KL_CTRL_RETURN = 10,
	KL_NUL = 0x103, KL_SHIFT_TAB = 0x10F,

	KL_CTRL_A =  1, KL_CTRL_B =  2, KL_CTRL_C =  3, KL_CTRL_D =  4,
	KL_CTRL_E =  5, KL_CTRL_F =  6, KL_CTRL_G =  7, KL_CTRL_H =  8,
	KL_CTRL_I =  9, KL_CTRL_J = 10, KL_CTRL_K = 11, KL_CTRL_L = 12,
	KL_CTRL_M = 13, KL_CTRL_N = 14, KL_CTRL_O = 15, KL_CTRL_P = 16,
	KL_CTRL_Q = 17, KL_CTRL_R = 18, KL_CTRL_S = 19, KL_CTRL_T = 20,
	KL_CTRL_U = 21, KL_CTRL_V = 22, KL_CTRL_W = 23, KL_CTRL_X = 24,
	KL_CTRL_Y = 25, KL_CTRL_Z = 26,

   KL_CTRL_PRTSC = 0x172,
   KL_CTRL_LEFT  = 0x173,
   KL_CTRL_RIGHT = 0x174,
   KL_CTRL_PGUP  = 0x184,
   KL_CTRL_END   = 0x175,
   KL_CTRL_PGDN  = 0x176,
   KL_CTRL_HOME  = 0x177,

   KL_ALT_A = 0x11e, KL_ALT_B = 0x130, KL_ALT_C = 0x12e, KL_ALT_D = 0x120,
   KL_ALT_E = 0x112, KL_ALT_F = 0x121, KL_ALT_G = 0x122, KL_ALT_H = 0x123,
   KL_ALT_I = 0x117, KL_ALT_J = 0x124, KL_ALT_K = 0x125, KL_ALT_L = 0x126,
   KL_ALT_M = 0x132, KL_ALT_N = 0x131, KL_ALT_O = 0x118, KL_ALT_P = 0x119,
   KL_ALT_Q = 0x110, KL_ALT_R = 0x113, KL_ALT_S = 0x11f, KL_ALT_T = 0x114,
   KL_ALT_U = 0x116, KL_ALT_V = 0x12f, KL_ALT_W = 0x111, KL_ALT_X = 0x12d,
   KL_ALT_Y = 0x115, KL_ALT_Z = 0x12c,

   KL_ALT_0 = 0x181, KL_ALT_1 = 0x178, KL_ALT_2 = 0x179, KL_ALT_3 = 0x17a,
   KL_ALT_4 = 0x17b, KL_ALT_5 = 0x17c, KL_ALT_6 = 0x17d, KL_ALT_7 = 0x17e,
   KL_ALT_8 = 0x17f, KL_ALT_9 = 0x180,

	KL_LEFT = 0x14b, KL_RIGHT = 0x14d,
	KL_UP   = 0x148, KL_DOWN  = 0x150,
   KL_HOME = 0x147, KL_END   = 0x14f,
	KL_PGUP = 0x149, KL_PGDN  = 0x151,
	KL_INS  = 0x152, KL_DEL   = 0x153,

   KL_F1 = 0x13b, KL_F2, KL_F3, KL_F4, KL_F5, KL_F6, KL_F7, KL_F8, KL_F9, KL_F0,
   KL_S1 = 0x154, KL_S2, KL_S3, KL_S4, KL_S5, KL_S6, KL_S7, KL_S8, KL_S9, KL_S0,
   KL_C1 = 0x15e, KL_C2, KL_C3, KL_C4, KL_C5, KL_C6, KL_C7, KL_C8, KL_C9, KL_C0,
   KL_A1 = 0x168, KL_A2, KL_A3, KL_A4, KL_A5, KL_A6, KL_A7, KL_A8, KL_A9, KL_A0,
};

#endif
/********************************************************************/
/* MATHINT: mathematic functions.                                   */
/********************************************************************/

#ifndef __MATHINT_H
#define __MATHINT_H

#ifdef __cplusplus
extern "C" {
#endif

int pascal bind(int l, int x, int h);
/* Truncates x into interval <l, h>. */
unsigned pascal ubind(unsigned l, unsigned x, unsigned h);
/* Truncates x into interval <l, h>. */
long pascal lbind(long l, long x, long h);
/* Truncates x into interval <l, h>. */
int pascal ilbind(int l, long x, int h);
/* Truncates x into interval <l, h>. */
int pascal isbound(int l, int x, int h);
/* Returns 1 when x is within interval <l,h>, 0 otherwise. */
int pascal isubound(unsigned l, unsigned x, unsigned h);
/* Returns 1 when x is within interval <l,h>, 0 otherwise. */
int pascal islbound(long l, long x, long h);
/* Returns 1 when x is within interval <l,h>, 0 otherwise. */
int pascal irgcmp(int l, int x, int h);
/* Returns -1 when x < l, 0 if x is within <l,h>, +1 when x > h. */
int pascal imin(int x, int y);
/* returns minimum of x, y. */
int pascal imax(int x, int y);
/* returns maximum of x, y. */
int pascal icmp(int x, int y);
/* returns -1 when x < y, 0 when x == y, 1 when x > y. */
unsigned pascal umin(unsigned x, unsigned y);
/* returns minimum of x, y. */
unsigned pascal umax(unsigned x, unsigned y);
/* returns maximum of x, y. */
int pascal ucmp(unsigned x, unsigned y);
/* returns -1 when x < y, 0 when x == y, 1 when x > y. */
long pascal lmin(long x, long y);
/* returns minimum of x, y. */
long pascal lmax(long x, long y);
/* returns maximum of x, y. */
int pascal lcmp(long x, long y);
/* returns -1 when x < y, 0 when x == y, 1 when x > y. */
long pascal iimul(int x, int y);
/* multiplies two integers into a long. */
unsigned long pascal uumul(unsigned x, unsigned y);
/* multiplies two unsigned integers into a long. */
int pascal lidiv(long x, int y);
/* divides a long by an integer. Errors are handled. */
int pascal uludiv(unsigned long x, unsigned y);
/* divides an unsigned long by an unsigned integer. Division by zero and */
/* overflow is handled. */
#define elemOf(a) (sizeof(a) / sizeof(*(a)))
#define half(x) ((x) >> 1) /* helps simplify expression syntax */
int pascal scale(int x, int m, int d);
/* Computes x * m / d; temporary result is a long. Errors are masked. */
int pascal uscale(unsigned x, unsigned m, unsigned d);
/* Computes x * m / d; temporary result is an unsigned long. On division */
/* by zero or overflow returns 0xFFFF. */
int pascal sdiv(int x, int y);
/* returns x / y when x % y is positive, or x / y - 1 when x % y is negative. */
int pascal smod(int x, int y);
/* Returns x % y. Always within interval <0, y). Y should be positive. */
int pascal ctoi(int x);
/* converts a character into a number. Converts digits and letters. Returns */
/* 255 on error. */
long pascal atoul(const void *s, void *endptr, int radix);
long pascal _fatoul(const void far *s, void far *far *endptr, int radix);
/* Converts string s into long in given radix; when <endptr> is not NULL, */
/* it is filled with pointer to 1st character following the processed part */
/* of the string. On error returns 0. s must point to 1st digit - no blank */
/* skipping is performed. */
unsigned long pascal isqr(int x);
/* Returns long x squared. */
unsigned pascal isqrt(unsigned long x);
/* Returns square root of x, i.e. largest of all number whose square is not */
/* greater than x (isqrt(1) = isqrt(3) = 1, isqrt(4) = isqrt(8) = 2 etc. */
int pascal mean(int x, int y);
/* Returns (x+y)/2 but always right (handles cases in which there would be */
/* normally an overflow). */
long pascal lmean(long x, long y);
/* Returns (x+y)/2, but always right (handles cases in which there would be */
/* normally an overflow. */
int pascal iabs(int x);
/* Returns absolute value of x. */
int pascal isgn(int x);
/* Returns -1 if x < 0, 0 if x == 0, 1 if x > 0. */
long pascal liabs(long x);
/* Returns absolute value of x. */
int pascal lisgn(long x);
/* returns -1 when x < 0, 0 when x == 0, 1 when x > 0. */
int pascal lscale(int x, long y, long z);
/* Returns x * y / z; temporary result is kept in a 6-byte. */
extern int pascal sin_table[];
/* table of 256 sines for angles 0 to 90 deg. Whole word range is used. */
int pascal qsin(int ang, int x);
/* Returns x times sin (ang); full angle (360 deg) is 1024. */
int pascal qcos(int ang, int x);
/* Returns x times cos (ang); full angle (360 deg) is 1024. */
int pascal ihyp(int x, int y);
/* Returns square root of x^2 + y^2. */
int pascal iarg(int x, int y);
/* Returns argument of vector [x,y]; 0 is to the east and full angle (360 */
/* deg) is 1024. When both x and y is 0, returns 0. */
long pascal xyVector(int x1, int y1, int x2, int y2);
/* Returns z-coordinate of vector product [x1,y1,0] x [x2,y2,0]. This is */
/* in fact also the value of determinant x1 * y2 - y1 * x2. */
long pascal xyScalar(int x1, int y1, int x2, int y2);
/* Returns scalar product [x1,y1] . [x2,y2], i.e. x1 * x2 + y1 * y2. */
unsigned pascal ufloor(unsigned x, unsigned y);
/* Returns maximum whole multiple of y that is <= x. */
unsigned pascal uceil(unsigned x, unsigned y);
/* Returns minimum whole multiple of y that is >= x. */
unsigned pascal usub(unsigned x, unsigned y);
/* Returns x - y when x > y and 0 when x <= y. */
long pascal iipow(int x, int y);
/* Returns x^y, where y should be positive. */
long pascal ipow10(int n);
/* Returns 10^n, where n should be positive. */
int pascal iswap(int x);
/* swaps MC68000-style bytes */
long pascal lswap(long x);
/* swaps MC68000-style bytes */
#ifdef __cplusplus
}
#endif

#endif
/********************************************************************/
/* WINTYPES.H: Windows-style definitions.                           */
/********************************************************************/

#if !defined(__WINTYPES__) && !defined(__WINDOWS_H)
#define __WINTYPES__

#define VOID                void

#define FAR                 far
#define NEAR                near
#define PASCAL              pascal
#define CDECL               cdecl

typedef int                 BOOL;
#define FALSE               0
#define TRUE                1

typedef unsigned char       BYTE;
typedef unsigned int        WORD;
typedef unsigned long       DWORD;

typedef unsigned int        UINT;

#ifdef STRICT
typedef signed long         LONG;
#else
#define LONG long
#endif

#undef  LOBYTE
#define LOBYTE(w)           ((BYTE)(w))
#undef  HIBYTE
#define HIBYTE(w)           ((BYTE)((UINT)(w) >> 8))

#undef  LOWORD
#define LOWORD(l)           ((WORD)(l))
#undef  HIWORD
#define HIWORD(l)           ((WORD)((DWORD)(l) >> 16))
#undef  LO
#define LO(l)               ((int)(l))
#undef  HI
#define HI(l)               ((int)((DWORD)(l) >> 16))

#undef  MAKELONG
#define MAKELONG(low, high) ((LONG)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))

#undef  max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#undef  min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#define __MINMAX_DEFINED

#undef  NULL
#define NULL 0

typedef const char FAR *LPCSTR;
typedef char *PSTR, NEAR *NPSTR, FAR *LPSTR, huge *HPSTR, _seg *SPSTR;

typedef BYTE  *PBYTE , NEAR *NPBYTE , FAR *LPBYTE , huge *HPBYTE , _seg *SPBYTE ;
typedef int   *PINT  , NEAR *NPINT  , FAR *LPINT  , huge *HPINT  , _seg *SPINT  ;
typedef WORD  *PWORD , NEAR *NPWORD , FAR *LPWORD , huge *HPWORD , _seg *SPWORD ;
typedef LONG  *PLONG , NEAR *NPLONG , FAR *LPLONG , huge *HPLONG , _seg *SPLONG ;
typedef DWORD *PDWORD, NEAR *NPDWORD, FAR *LPDWORD, huge *HPDWORD, _seg *SPDWORD;
typedef void  *PVOID , NEAR *NPVOID , FAR *LPVOID, _seg *SPVOID;

#undef  MAKELP
#define MAKELP(sel, off)    ((void FAR*)MAKELONG((off), (sel)))
#undef  SELECTOROF
#define SELECTOROF(lp)      HIWORD(lp)
#undef  OFFSETOF
#define OFFSETOF(lp)        LOWORD(lp)

typedef struct tagPOINT
{
    int x;
    int y;
} POINT, *PPOINT, FAR *LPPOINT;

#undef  FIELDOFFSET
#define FIELDOFFSET(type, field)    ((int)(&((type NEAR*)1)->field)-1)

#endif
